require_relative 'funding_round'
require 'csv'

module Crowdfund
  class Incubator
    attr_reader :name

    def initialize(name)
      @name = name
      @projects = []
    end

    def load_projects(from_file)
      CSV.foreach(from_file) do |row|
        project = Project.new(row[0], row[1].to_i, row[2].to_i)
        add_project(project)
      end
    end

    def save_projects(to_file='projects_funding.csv')
      CSV.open(to_file, 'wb') do |csv|
        @projects.sort.each do |project|
          csv << [project.name, project.target_funding, project.total_funding]
        end
      end
      # File.open(to_file, 'w') do |file|
      #   @projects.sort.each do |project|
      #     file.puts "#{project.name},#{project.target_funding},#{project.total_funding}"
      #   end
      # end
    end

    def add_project(project)
      @projects << project
    end

    def total_pledge_amount
      @projects.reduce(0) { |sum, project| sum += project.total_pledges }
    end

    def total_funding_amount
      @projects.reduce(0) { |sum, project| sum += project.total_funding }
    end

    def request_funding(rounds)
      puts "There are #{@projects.size} projects in #{@name} that need funding:"
      puts @projects

      pledges = PledgePool::PLEDGES
      puts "\nThere are #{pledges.size} possible pledge amounts:"
      pledges.each do |pledge|
        puts "\tA #{pledge.name} pledge is worth $#{pledge.amount}."
      end

      1.upto(rounds) do |round|
        puts "\nRound #{round}:"
        @projects.each do |project|
          FundingRound.pitch_for_funds(project)
          puts project
        end
      end
    end

    def print_project_name_and_funding(project)
      puts "#{project.name} ended with $#{project.total_funding} in funding."
    end

    def status_of_projects
      fully_funded, under_funded = @projects.partition { |p| p.funded? }

      puts "\n#{@name}'s final funding amounts:"
      puts "$#{total_funding_amount}"

      puts "\n#{@name}'s total pledges:"
      puts "$#{total_pledge_amount}"

      @projects.sort.each do |project|
        puts "\n#{project.name} pledges:"
        project.each_pledge do |pledge|
          puts "$#{pledge.amount} in #{pledge.name} pledges"
        end
        puts "$#{project.total_pledges} in total pledges"
      end

      puts "\nFully Funded Projects:"
      fully_funded.each do |project|
        print_project_name_and_funding(project)
      end

      puts "\nUnder Funded Projects:"
      under_funded.each do |project|
        print_project_name_and_funding(project)
      end

      puts "\nStill in need of funding:"
      under_funded.sort.each do |project|
        formatted_name = project.name.ljust(50, '.')
        puts "#{formatted_name} $#{project.funding_needed} needed."
      end
    end
  end
end

if __FILE__ == $0
  project = Project.new('Leather Goods', 5000)
  incubator = Incubator.new('YC')
  incubator.add_project(project)
  incubator.request_funding(2)
end
