require_relative 'incubator'

module Crowdfund
  describe Incubator do

    before do
      $stdout = StringIO.new
      @incubator = Incubator.new('YC')
    end

    context "with one project" do

      before do
        @funding = 500
        @project = Project.new('Leather Goods', 5000, @funding)

        @incubator.add_project(@project)
      end

      it "should add funds if an even number is rolled" do
        allow_any_instance_of(Die).to receive(:roll).and_return(2)

        @incubator.request_funding(3)

        expect(@project.funding).to eq(@funding + (2000 * 3))
      end

      it "should remove funds if an odd number is rolled" do
        allow_any_instance_of(Die).to receive(:roll).and_return(1)

        @incubator.request_funding(3)

        expect(@project.funding).to eq(@funding - (1000 * 3))
      end
    end

    context "with more than one project" do

      before do
        @project1 = Project.new('Leather Goods', 1000)
        @project2 = Project.new('American Watches', 1000)

        @incubator.add_project(@project1)
        @incubator.add_project(@project2)
      end

      it "calculates the total pledge amounts for all projects" do
        @project1.accept_pledge(Pledge.new(:gold, 100))
        @project2.accept_pledge(Pledge.new(:gold, 100))

        expect(@incubator.total_pledge_amount).to eq(200)
      end
    end
  end
end
