require_relative 'fundable'

module Crowdfund
  class Project
    include Fundable

    attr_accessor :name, :target_funding, :funding

    def initialize(name, target_funding, funding=500)
      @name = name
      @target_funding = target_funding
      @funding = funding
      @pledges = Hash.new(0)
    end

    def each_pledge
      @pledges.each do |name, amount|
        yield(Pledge.new(name, amount))
      end
    end

    def accept_pledge(pledge)
      @pledges[pledge.name] += pledge.amount
      puts "#{@name} received a #{pledge.name} pledge worth $#{pledge.amount}"
      puts "#{@name}'s pledges: #{@pledges}"
    end

    def total_pledges
      @pledges.values.reduce(0, :+)
    end

    def total_funding
      @funding + total_pledges
    end

    def to_s
      "#{@name} current funding is $#{total_funding} with $#{funding_needed} needed to reach its target funding amount."
    end
  end
end

if __FILE__ == $0
  project = Project.new('Leather Goods', 5000)
  puts project
  project.add_funds
  puts project.funding
  project.remove_funds
  puts project.funding
  project.add_funds(250)
  puts project.funding
  project.remove_funds(150)
  puts project.funding
  puts project.funding_needed
  puts project
end
